@@ -42,6 +42,8 @@ import time
 from pathlib import Path
 from typing import Any, Dict, Optional
 
+import pyperclip
+
 try:
     import readline
 
@@ -72,6 +74,7 @@ from .components import (
     ConfigWizard,
     DependencyManager,
     DisplayManager,
+    SessionManager,
     StatusBar,
     TemplateManager,
     TokenTracker,
@@ -405,18 +408,28 @@ class ChatLoop:
         agent_name: str,
         agent_description: str,
         agent_factory=None,
+        agent_path: Optional[str] = None,
         config: Optional["ChatConfig"] = None,
     ):
         self.agent = agent
         self.agent_name = agent_name
         self.agent_description = agent_description
         self.agent_factory = agent_factory  # Function to create fresh agent instance
+        self.agent_path = agent_path or "unknown"  # Store for session metadata
         self.history_file = None
         self.last_response = ""  # Track last response for copy command
+        self.last_query = ""  # Track last user query for copy command
 
         # Conversation tracking for auto-save
         self.conversation_history: list[Dict[str, Any]] = []
 
+        # Generate session ID for this chat session
+        from datetime import datetime
+
+        timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
+        safe_agent_name = agent_name.lower().replace(" ", "_").replace("/", "_")
+        self.session_id = f"{safe_agent_name}_{timestamp}"
+
         # Load or use provided config
         self.config = config if config else get_config()
 
@@ -570,6 +583,16 @@ class ChatLoop:
             enabled=audio_enabled, sound_file=audio_sound_file
         )
 
+        # Setup session manager for conversation persistence
+        sessions_dir = (
+            self.config.expand_path(
+                self.config.get("paths.save_location", "~/agent-conversations")
+            )
+            if self.config
+            else Path.home() / "agent-conversations"
+        )
+        self.session_manager = SessionManager(sessions_dir=sessions_dir)
+
     def _extract_token_usage(self, response_obj) -> Optional[Dict[str, int]]:
         """
         Extract token usage from response object.
@@ -660,9 +683,230 @@ class ChatLoop:
 
         return None
 
+    def _extract_code_blocks(self, text: str) -> list:
+        """
+        Extract code blocks from markdown text.
+
+        Args:
+            text: Markdown text containing code blocks
+
+        Returns:
+            List of code block contents (without fence markers)
+        """
+        import re
+
+        # Match code blocks with triple backticks
+        pattern = r"```(?:\w+)?\n(.*?)```"
+        matches = re.findall(pattern, text, re.DOTALL)
+        return [match.strip() for match in matches]
+
+    def _format_conversation_as_markdown(self) -> str:
+        """
+        Format entire conversation history as markdown.
+
+        Returns:
+            Markdown-formatted conversation
+        """
+        from datetime import datetime
+
+        lines = []
+        lines.append(f"# {self.agent_name} - Conversation")
+        lines.append(f"\nSession ID: {self.session_id}")
+        lines.append(f"Agent: {self.agent_name}")
+
+        if self.conversation_history:
+            first_ts = self.conversation_history[0]["timestamp"]
+            lines.append(f"Started: {datetime.fromtimestamp(first_ts)}")
+
+        lines.append("\n---\n")
+
+        for i, entry in enumerate(self.conversation_history, 1):
+            timestamp = datetime.fromtimestamp(entry["timestamp"]).strftime(
+                "%H:%M:%S"
+            )
+            lines.append(f"\n## Query {i} ({timestamp})\n")
+            lines.append(f"**You:** {entry['query']}\n")
+            lines.append(f"**{self.agent_name}:**\n\n{entry['response']}\n")
+
+            if entry.get("duration"):
+                duration = entry["duration"]
+                lines.append(f"\n*Response time: {duration:.1f}s*")
+
+            if entry.get("usage"):
+                usage = entry["usage"]
+                input_tok = usage.get("input_tokens", 0)
+                output_tok = usage.get("output_tokens", 0)
+                total = input_tok + output_tok
+                lines.append(
+                    f" | *Tokens: {total:,} (in: {input_tok:,}, out: {output_tok:,})*"
+                )
+
+            lines.append("\n---\n")
+
+        return "\n".join(lines)
+
+    async def restore_session(self, session_id: str) -> bool:
+        """
+        Restore a previous session by replaying conversation to agent.
+
+        Args:
+            session_id: Session ID or number from sessions list
+
+        Returns:
+            True if session was successfully restored, False otherwise
+        """
+        try:
+            # If session_id is a number, resolve it from the list
+            if session_id.isdigit():
+                session_num = int(session_id)
+                sessions = self.session_manager.list_sessions(
+                    agent_name=self.agent_name, limit=20
+                )
+
+                if session_num < 1 or session_num > len(sessions):
+                    print(Colors.error(f"Invalid session number: {session_num}"))
+                    print(f"Valid range: 1-{len(sessions)}")
+                    return False
+
+                # Get actual session_id from the list (1-indexed)
+                session_info = sessions[session_num - 1]
+                session_id = session_info.session_id
+            else:
+                # Get session metadata for display
+                session_info = self.session_manager.get_session_metadata(session_id)
+                if not session_info:
+                    print(Colors.error(f"Session not found: {session_id}"))
+                    return False
+
+            # Load the session data
+            session_data = self.session_manager.load_session(session_id)
+            if not session_data:
+                print(Colors.error(f"Could not load session: {session_id}"))
+                return False
+
+            conversation = session_data.get("conversation", [])
+            if not conversation:
+                print(Colors.error("Session has no conversation history"))
+                return False
+
+            # Check if agent matches
+            if session_data["agent_name"] != self.agent_name:
+                print(
+                    Colors.system(
+                        f"⚠️  Warning: Session was created with "
+                        f"'{session_data['agent_name']}' "
+                        f"but you're using '{self.agent_name}'"
+                    )
+                )
+                confirm = input(Colors.system("Continue anyway? (y/n): "))
+                if confirm.lower() != "y":
+                    print(Colors.system("Resume cancelled"))
+                    return False
+
+            # Display what we're loading
+            print(f"\n🔄 Loading session: {session_id}")
+            print(f"   Agent: {session_data['agent_name']}")
+            print(f"   Queries: {len(conversation)}")
+            print(f"   Created: {session_info.created.strftime('%b %d, %Y at %H:%M')}")
+            if session_info.total_cost > 0:
+                print(f"   Cost so far: ${session_info.total_cost:.3f}")
+            print()
+
+            # Ask for confirmation (if enabled in config)
+            resume_confirmation = (
+                self.config.get("sessions.resume_confirmation", True)
+                if self.config
+                else True
+            )
+
+            if resume_confirmation:
+                print(
+                    Colors.system(
+                        "⚠️  This will restore conversation context to the agent."
+                    )
+                )
+                confirm = input(Colors.system("Continue? (y/n): "))
+                if confirm.lower() != "y":
+                    print(Colors.system("Resume cancelled"))
+                    return False
+
+            # Replay strategy: Silent replay of all queries
+            print(Colors.system(f"\nReplaying {len(conversation)} previous queries..."))
+            print(Colors.system("(This restores the agent's context)"))
+
+            replay_start = time.time()
+
+            for i, entry in enumerate(conversation, 1):
+                query = entry["query"]
+                # Display progress every 5 queries
+                if i % 5 == 0 or i == len(conversation):
+                    print(
+                        Colors.system(f"  Progress: {i}/{len(conversation)} queries"),
+                        end="\r",
+                    )
+
+                try:
+                    # Replay query silently (don't display response)
+                    # Check if agent supports streaming
+                    if hasattr(self.agent, "stream_async"):
+                        # Consume the stream but don't display
+                        async for _ in self.agent.stream_async(query):
+                            pass
+                    else:
+                        # Non-streaming agent
+                        await asyncio.get_event_loop().run_in_executor(
+                            None, self.agent, query
+                        )
+
+                except Exception as e:
+                    logger.warning(f"Error replaying query {i}: {e}")
+                    # Continue with next query
+
+            replay_duration = time.time() - replay_start
+            print()  # Clear progress line
+            print(
+                Colors.success(
+                    f"✓ Replayed {len(conversation)} queries in {replay_duration:.1f}s"
+                )
+            )
+
+            # Restore conversation history
+            self.conversation_history = conversation.copy()
+
+            # Update session ID to continue this session
+            self.session_id = session_id
+
+            # Restore token tracker state
+            if session_info.total_tokens > 0:
+                # Approximate token restoration (not exact, but close)
+                # We can't perfectly restore because we don't have per-query tokens
+                # But we can set the total
+                self.token_tracker.total_input_tokens = int(
+                    session_info.total_tokens * 0.6
+                )
+                self.token_tracker.total_output_tokens = int(
+                    session_info.total_tokens * 0.4
+                )
+
+            # Update query count
+            self.query_count = len(conversation)
+
+            # Display confirmation
+            self.display_manager.display_session_loaded(session_info, len(conversation))
+
+            logger.info(f"Successfully restored session: {session_id}")
+            return True
+
+        except Exception as e:
+            logger.error(f"Failed to restore session: {e}", exc_info=True)
+            print(Colors.error(f"\n⚠️  Failed to restore session: {e}"))
+            return False
+
     def save_conversation(self) -> bool:
         """
-        Save conversation history to markdown file.
+        Save conversation history using SessionManager.
+
+        Saves both JSON (for resume) and markdown (for humans) formats.
 
         Returns:
             True if save was successful, False otherwise
@@ -673,82 +917,36 @@ class ChatLoop:
             return False
 
         try:
-            # Get save location from config or use default
-            save_location = (
-                self.config.expand_path(
-                    self.config.get("paths.save_location", "~/agent-conversations")
-                )
-                if self.config
-                else Path.home() / "agent-conversations"
+            # Calculate total cost for metadata
+            total_cost = self.token_tracker.get_cost()
+            session_duration = time.time() - self.session_start_time
+
+            metadata = {
+                "total_cost": total_cost,
+                "duration": session_duration,
+            }
+
+            # Use SessionManager to save
+            success, message = self.session_manager.save_session(
+                session_id=self.session_id,
+                agent_name=self.agent_name,
+                agent_path=self.agent_path,
+                agent_description=self.agent_description,
+                conversation=self.conversation_history,
+                metadata=metadata,
             )
 
-            # Ensure directory exists
-            save_location.mkdir(parents=True, exist_ok=True)
-
-            # Generate filename with timestamp
-            from datetime import datetime
-
-            timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
-            safe_agent_name = self.agent_name.lower()
-            safe_agent_name = safe_agent_name.replace(" ", "_").replace("/", "_")
-            filename = f"{safe_agent_name}_{timestamp}.md"
-            filepath = save_location / filename
-
-            # Format conversation as markdown
-            content_lines = [
-                f"# {self.agent_name} Conversation",
-                f"\n**Date:** {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}",
-                f"\n**Agent:** {self.agent_name}",
-                f"\n**Description:** {self.agent_description}",
-                f"\n**Total Queries:** {len(self.conversation_history)}",
-                "\n---\n",
-            ]
-
-            # Add each conversation entry
-            for i, entry in enumerate(self.conversation_history, 1):
-                ts = entry["timestamp"]
-                entry_timestamp = datetime.fromtimestamp(ts).strftime("%H:%M:%S")
-                duration = entry.get("duration", 0)
-
-                # Add query
-                content_lines.append(f"\n## Query {i} ({entry_timestamp})\n")
-                content_lines.append(f"**You:** {entry['query']}\n")
-
-                # Add response
-                content_lines.append(f"\n**{self.agent_name}:** {entry['response']}\n")
-
-                # Add metadata
-                metadata_parts = [f"Time: {duration:.1f}s"]
-
-                # Add token info if available
-                usage = entry.get("usage")
-                if usage and usage is not None:
-                    input_tok = usage.get("input_tokens", 0)
-                    output_tok = usage.get("output_tokens", 0)
-                    total_tok = input_tok + output_tok
-                    if total_tok > 0:
-                        tok_str = f"Tokens: {total_tok:,} "
-                        tok_str += f"(in: {input_tok:,}, out: {output_tok:,})"
-                        metadata_parts.append(tok_str)
-
-                if metadata_parts:
-                    content_lines.append(f"\n*{' | '.join(metadata_parts)}*\n")
-
-                content_lines.append("\n---\n")
-
-            # Write to file
-            with open(filepath, "w", encoding="utf-8") as f:
-                f.writelines(content_lines)
-
-            # Set secure permissions (readable/writable by owner only)
-            filepath.chmod(0o600)
-
-            logger.info(f"Conversation saved to: {filepath}")
-            print(Colors.success(f"\n💾 Conversation saved to: {filepath}"))
-            return True
+            if success:
+                logger.info(f"Conversation saved: {self.session_id}")
+                print(Colors.success(f"\n💾 {message}"))
+                return True
+            else:
+                logger.warning(f"Failed to save conversation: {message}")
+                print(Colors.error(f"\n⚠️  {message}"))
+                return False
 
         except Exception as e:
-            logger.warning(f"Failed to save conversation: {e}")
+            logger.warning(f"Failed to save conversation: {e}", exc_info=True)
             print(Colors.error(f"\n⚠️  Could not save conversation: {e}"))
             return False
 
@@ -1245,6 +1443,50 @@ class ChatLoop:
 
         self.display_manager.display_banner()
 
+        # Handle --resume flag if specified
+        if hasattr(self, "resume_session_ref") and self.resume_session_ref:
+            session_ref = self.resume_session_ref
+
+            # If "pick" mode, show session picker
+            if session_ref == "pick":
+                sessions = self.session_manager.list_sessions(
+                    agent_name=self.agent_name, limit=20
+                )
+
+                if not sessions:
+                    print(Colors.system("\nNo saved sessions found to resume."))
+                    print("Continue with fresh session...\n")
+                else:
+                    self.display_manager.display_sessions(
+                        sessions, agent_name=self.agent_name
+                    )
+                    print()
+                    try:
+                        prompt = (
+                            "Enter session number to resume (or press Enter to skip): "
+                        )
+                        choice = input(Colors.system(prompt)).strip()
+
+                        if choice:
+                            success = await self.restore_session(choice)
+                            if success:
+                                # Clear and redisplay banner after resume
+                                self.display_manager.display_banner()
+                    except (KeyboardInterrupt, EOFError):
+                        print()
+                        print(
+                            Colors.system("Skipping resume, starting fresh session...")
+                        )
+                        print()
+            else:
+                # Direct resume with specific session ID/number
+                success = await self.restore_session(session_ref)
+                if success:
+                    # Clear and redisplay banner after resume
+                    self.display_manager.display_banner()
+                else:
+                    print(Colors.system("\nContinuing with fresh session...\n"))
+
         try:
             while True:
                 try:
@@ -1275,6 +1517,93 @@ class ChatLoop:
                             templates, self.prompts_dir
                         )
                         continue
+                    elif user_input.lower() == "sessions":
+                        # List saved sessions
+                        sessions = self.session_manager.list_sessions(
+                            agent_name=self.agent_name, limit=20
+                        )
+                        self.display_manager.display_sessions(
+                            sessions, agent_name=self.agent_name
+                        )
+                        continue
+                    elif user_input.lower().startswith("copy"):
+                        # Copy command with variants
+                        parts = user_input.lower().split(maxsplit=1)
+                        copy_mode = parts[1] if len(parts) > 1 else ""
+
+                        try:
+                            content = None
+                            description = ""
+
+                            if copy_mode == "query":
+                                # Copy last user query
+                                if self.last_query:
+                                    content = self.last_query
+                                    description = "last query"
+                                else:
+                                    print(Colors.system("No query to copy yet"))
+                                    continue
+                            elif copy_mode == "all":
+                                # Copy entire conversation as markdown
+                                if self.conversation_history:
+                                    content = self._format_conversation_as_markdown()
+                                    description = "entire conversation"
+                                else:
+                                    print(Colors.system("No conversation to copy yet"))
+                                    continue
+                            elif copy_mode == "code":
+                                # Copy just code blocks from last response
+                                if self.last_response:
+                                    code_blocks = self._extract_code_blocks(
+                                        self.last_response
+                                    )
+                                    if code_blocks:
+                                        content = "\n\n".join(code_blocks)
+                                        description = "code blocks from last response"
+                                    else:
+                                        print(
+                                            Colors.system(
+                                                "No code blocks found in last response"
+                                            )
+                                        )
+                                        continue
+                                else:
+                                    print(Colors.system("No response to copy yet"))
+                                    continue
+                            else:
+                                # Default: copy last response
+                                if self.last_response:
+                                    content = self.last_response
+                                    description = "last response"
+                                else:
+                                    print(Colors.system("No response to copy yet"))
+                                    continue
+
+                            # Copy to clipboard
+                            pyperclip.copy(content)
+                            print(
+                                Colors.success(f"✓ Copied {description} to clipboard")
+                            )
+
+                        except Exception as e:
+                            print(Colors.error(f"Failed to copy: {e}"))
+                            logger.error(f"Copy command failed: {e}")
+                        continue
+                    elif user_input.lower().startswith("resume "):
+                        # Resume a previous session
+                        parts = user_input.split(maxsplit=1)
+                        if len(parts) < 2:
+                            print(Colors.error("Usage: resume <session_id or number>"))
+                            print("Use 'sessions' command to see available sessions")
+                            continue
+
+                        session_ref = parts[1].strip()
+                        success = await self.restore_session(session_ref)
+
+                        if success:
+                            # Show banner after resume
+                            self.display_manager.display_banner()
+                        continue
                     elif user_input.startswith("/") and len(user_input) > 1:
                         # Template command: /template_name <optional input>
                         parts = user_input[1:].split(maxsplit=1)
@@ -1350,6 +1679,9 @@ class ChatLoop:
                     # Process query through agent
                     logger.info(f"Processing query: {user_input[:100]}...")
 
+                    # Track query for copy command
+                    self.last_query = user_input
+
                     # Update terminal title to show processing
                     if self.update_terminal_title:
                         set_terminal_title(f"{self.agent_name} - Processing...")
@@ -1508,6 +1840,27 @@ Examples:
         ),
     )
 
+    # Session management
+    session_group = parser.add_argument_group("session management")
+
+    session_group.add_argument(
+        "--resume",
+        "-r",
+        nargs="?",
+        const="pick",
+        metavar="SESSION",
+        help=(
+            "Resume a previous session (optionally specify session ID or number, "
+            "otherwise shows picker)"
+        ),
+    )
+
+    session_group.add_argument(
+        "--list-sessions",
+        action="store_true",
+        help="List all saved sessions and exit",
+    )
+
     # Configuration wizard
     parser.add_argument(
         "--wizard",
@@ -1587,6 +1940,43 @@ Examples:
             print(Colors.error(message))
             sys.exit(1)
 
+    # Handle session management commands
+    if args.list_sessions:
+        # List sessions and exit
+        config = get_config(config_path=args.config) if args.config else get_config()
+        sessions_dir = (
+            config.expand_path(
+                config.get("paths.save_location", "~/agent-conversations")
+            )
+            if config
+            else Path.home() / "agent-conversations"
+        )
+
+        session_manager = SessionManager(sessions_dir=sessions_dir)
+        sessions = session_manager.list_sessions(limit=50)
+
+        if sessions:
+            print(f"\n{Colors.system('Saved Sessions')} ({len(sessions)}):")
+            print(f"{Colors.DIM}{'-' * 60}{Colors.RESET}")
+            for i, session in enumerate(sessions, 1):
+                created_str = session.created.strftime("%b %d, %Y %H:%M")
+                session_line = f"  {i:2}. {session.agent_name:<20} {created_str}"
+                session_line += f"  {session.query_count:3} queries"
+                if session.total_cost > 0:
+                    session_line += f"  ${session.total_cost:.2f}"
+                print(session_line)
+                preview_text = f'      "{session.preview}"'
+                print(f"{Colors.DIM}{preview_text}{Colors.RESET}")
+
+            print(f"{Colors.DIM}{'-' * 60}{Colors.RESET}")
+            print(f"\nResume: {Colors.system('chat_loop <agent> --resume <number>')}")
+            print(f"        {Colors.system('chat_loop <agent> --resume <session_id>')}")
+        else:
+            print(f"\n{Colors.system('No saved sessions found')}")
+            print(f"Sessions will be saved to: {sessions_dir}")
+
+        sys.exit(0)
+
     # Require agent argument if not doing alias management
     if not args.agent:
         print(Colors.error("Error: Agent path or alias required"))
@@ -1699,8 +2089,17 @@ Examples:
             agent_name,
             agent_description,
             agent_factory=create_fresh_agent,
+            agent_path=str(agent_path),
             config=config,
         )
+
+        # Handle --resume flag if provided
+        if args.resume:
+            # Store resume session for processing in _async_run
+            chat_loop.resume_session_ref = args.resume
+        else:
+            chat_loop.resume_session_ref = None
+
         chat_loop.run()
 
     except Exception as e:
